# third-party imprts
import json

import click
import uvicorn
from rich import print

# app imports
from sneakpeek.server import get_link_preview


@click.group()
def cli():
    pass


@click.command()
@click.option("--host", default="localhost", help="Sever Host.")
@click.option("--port", default=9000, help="Server Port.")
@click.option("--reload", is_flag=True, help="Enable auto-reload on code change.")
@click.option("--workers", default=2, help="No. of worker threads.")
def serve(host, port, reload, workers):
    """Spin up a simple server using FastAPI."""
    print(
        f"Starting a server on {host}:{port} "
        f"with auto reload {reload} and {workers} workers"
    )
    uvicorn.run(
        "sneakpeek.server:app", host=host, port=port, reload=reload, workers=workers
    )


@click.command()
@click.option("--url", required=True, help="URL to be generate a link preview for.")
@click.option(
    "--scrape",
    is_flag=True,
    help="Try scraping website if open graph tags are not present.",
)
def preview(url, scrape):
    """Link preview the command line."""
    print(json.dumps(get_link_preview(url=url, scrape=scrape), indent=4))


cli.add_command(serve)
cli.add_command(preview)



# encoding: utf-8

"""Module for SneakPeek."""

# standard imports
import json
import os
import re
import urllib.request
from urllib.parse import urlparse

# third-party imports
import tweepy
import validators
from bs4 import BeautifulSoup

consumer_key = os.environ.get("TWITTER_CONSUMER_KEY", "")
consumer_secret = os.environ.get("TWITTER_CONSUMER_SECRET", "")
access_token = os.environ.get("TWITTER_ACCESS_TOKEN", "")
access_token_secret = os.environ.get("TWITTER_ACCESS_TOKEN_SECRET", "")
auth = tweepy.OAuth1UserHandler(consumer_key, consumer_secret)
auth.set_access_token(access_token, access_token_secret)
api = tweepy.API(auth)
try:
    api.verify_credentials()
    TWITTER_SETUP = True
except:
    TWITTER_SETUP = False


class SneakPeek(dict):
    """ """

    required_attrs = ["title", "type", "image", "url", "description"]

    def __init__(self, url=None, html=None, scrape=False, **kwargs):
        # If scrape == True, then will try to fetch missing attribtues
        # from the page's body
        self.domain = None
        self.scrape = scrape
        self.url = url

        self.title = ""
        self.type = ""
        self.image = ""
        self.description = ""

        self.error = None

        for k in kwargs.keys():
            self[k] = kwargs[k]

        if url:
            self.is_valid_url(url)
            self.domain = urlparse(self.url).netloc

        if html is not None:
            self.parse(html)

    def __setattr__(self, name, val):
        self[name] = val

    def __getattr__(self, name):
        return self[name]

    def is_valid_url(self, url=None):
        return validators.url(url)

    def is_twitter_setup(self):
        global TWITTER_SETUP
        return TWITTER_SETUP

    def fetch_and_parse_twitter(self):
        if not self.is_twitter_setup():
            return
        if "status" in self.url:
            tweet_id = int(self.url.split("status/")[-1].split("?")[0])
            status = api.get_status(tweet_id, tweet_mode="extended")
            self.title = f"{status.user.name} on Twitter"
            self.description = status.full_text
            if status._json["entities"].get("media"):
                self.image = status._json["entities"]["media"][0]["media_url_https"]
            else:
                self.image = status._json["user"]["profile_image_url_https"].replace(
                    "_normal", ""
                )
            self.type = "article"
            self.created_at = status._json["created_at"]
            return
        else:
            username = self.url.strip("/").split("/")[-1]
            user = api.get_user(screen_name=username)
            self.title = f"{user.name} (@{user.screen_name}) / Twitter"
            self.type = "profile"
            self.description = user.description
            self.image = user.profile_image_url_https.replace("_normal", "")

    def fetch(self):
        """ """
        # TODO: use random user agent for every request - https://github.com/Luqman-Ud-Din/random_user_agent
        if self.domain == "twitter.com":
            return self.fetch_and_parse_twitter()
        req = urllib.request.Request(self.url, headers={"User-Agent": "Mozilla/5.0"})
        try:
            with urllib.request.urlopen(req) as raw:
                html = raw.read()
        except Exception as err:
            html = ""
            self.error = str(err)
            return
        return self.parse(html)

    def parse(self, html):
        """ """
        if isinstance(html, BeautifulSoup):
            doc = html
        else:
            doc = BeautifulSoup(html, "html.parser")
        try:
            ogs = doc.html.head.findAll(property=re.compile(r"^og"))
        except:
            self.error = "Parsing Error: Open Graph Meta Not Found"
            ogs = []
        for og in ogs:
            if og.has_attr("content"):
                self[og["property"][3:]] = og["content"]
        # Couldn't fetch all attrs from og tags, try scraping body
        if not self.is_valid() and self.scrape:
            for attr in self.required_attrs:
                if not self.valid_attr(attr):
                    try:
                        self[attr] = getattr(self, "scrape_%s" % attr)(doc)
                    except AttributeError:
                        pass

    def valid_attr(self, attr):
        return self.get(attr) and len(self[attr]) > 0

    def is_valid(self):
        return all([self.valid_attr(attr) for attr in self.required_attrs])

    def to_html(self):
        if not self.is_valid():
            return '<meta property="og:error" content="og metadata is not valid" />'

        meta = ""
        for key, value in self.iteritems():
            meta += '\n<meta property="og:%s" content="%s" />' % (key, value)
        meta += "\n"

        return meta

    def to_json(self):
        if not self.is_valid():
            return json.dumps({"error": "og metadata is not valid"})

        return json.dumps(self)

    def to_xml(self):
        pass

    def scrape_image(self, doc):
        images = [dict(img.attrs)["src"] for img in doc.html.body.findAll("img")]

        if images:
            return images[0]

        return ""

    def scrape_title(self, doc):
        try:
            return doc.html.head.title.text
        except:
            return ""

    def scrape_type(self, doc):
        try:
            return "other"
        except:
            return ""

    def scrape_url(self, doc):
        try:
            return self.url
        except:
            return ""

    def scrape_description(self, doc):
        try:
            tag = doc.html.head.findAll("meta", attrs={"name": "description"})
            result = "".join([t["content"] for t in tag])
            return result
        except:
            return ""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sneakpeek']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'click>=8.1.3,<9.0.0',
 'fastapi>=0.79.0,<0.80.0',
 'rich>=12.5.1,<13.0.0',
 'tweepy>=4.10.0,<5.0.0',
 'uvicorn>=0.18.2,<0.19.0',
 'validators>=0.20.0,<0.21.0']

entry_points = \
{'console_scripts': ['sneakpeek = sneakpeek.console:cli']}

setup_kwargs = {
    'name': 'sneakpeek',
    'version': '0.8.0',
    'description': 'A python module to generate link previews.',
    'long_description': '\n<div align="center">\n  <h1>\n    SneakPeek\n  </h1>\n  <h4>A python module and a minimalistic server to generate link previews.</h4>\n</div>\n\n[![License: MIT](https://img.shields.io/badge/License-MIT-green.svg)](https://opensource.org/licenses/MIT)\n[![Downloads](https://pepy.tech/badge/sneakpeek)](https://pepy.tech/project/sneakpeek)\n[![PyPI](https://img.shields.io/pypi/v/sneakpeek)](https://pypi.org/project/sneakpeek)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/sneakpeek)](https://pypi.org/project/sneakpeek)\n[![test-ci](https://img.shields.io/github/workflow/status/codingcoffee/sneakpeek/test-ci)](https://github.com/codingCoffee/sneakpeek/actions)\n[![Docker Pulls](https://img.shields.io/docker/pulls/codingcoffee/sneakpeek)](https://hub.docker.com/r/codingcoffee/sneakpeek)\n[![Docker Image Size (tag)](https://img.shields.io/docker/image-size/codingcoffee/sneakpeek/latest)](https://hub.docker.com/r/codingcoffee/sneakpeek)\n\n## What is supported\n\n- Any page which supports [Open Graph Protocol](https://ogp.me) (which most sane websites do)\n- Special handling for sites like\n  - [Twitter](https://twitter.com) (requires a twitter [API key](https://developer.twitter.com/))\n\n\n## Installation\n\nRun the following to install\n\n```sh\npip install sneakpeek\n```\n\n\n## Usage as a Python Module\n\n### From a URL\n\n```sh\n>>> import sneakpeek\n>>> from pprint import pprint\n\n>>> link = sneakpeek.SneakPeek("https://www.youtube.com/watch?v=dQw4w9WgXcQ")\n>>> link.fetch()\n>>> link.is_valid()\nTrue\n>>> pprint(link)\n{\'description\': \'The official video for “Never Gonna Give You Up” by Rick \'\n                \'AstleyTaken from the album ‘Whenever You Need Somebody’ – \'\n                \'deluxe 2CD and digital deluxe out 6th May ...\',\n \'domain\': \'www.youtube.com\',\n \'image\': \'https://i.ytimg.com/vi/dQw4w9WgXcQ/maxresdefault.jpg\',\n \'image:height\': \'720\',\n \'image:width\': \'1280\',\n \'scrape\': False,\n \'site_name\': \'YouTube\',\n \'title\': \'Rick Astley - Never Gonna Give You Up (Official Music Video)\',\n \'type\': \'video.other\',\n \'url\': \'https://www.youtube.com/watch?v=dQw4w9WgXcQ\',\n \'video:height\': \'720\',\n \'video:secure_url\': \'https://www.youtube.com/embed/dQw4w9WgXcQ\',\n \'video:tag\': \'never gonna give you up karaoke\',\n \'video:type\': \'text/html\',\n \'video:url\': \'https://www.youtube.com/embed/dQw4w9WgXcQ\',\n \'video:width\': \'1280\'}\n\n>>> link = sneakpeek.SneakPeek(url="https://codingcoffee.dev")\n>>> link.fetch()\n>>> pprint(link)\n{\'description\': \'A generalist with multi faceted interests and extensive \'\n                \'experience with DevOps, System Design and Full Stack \'\n                \'Development. I like blogging about things which interest me, \'\n                \'have a niche for optimizing and customizing things to the \'\n                \'very last detail, this includes my text editor and operating \'\n                \'system alike.\',\n \'domain\': \'codingcoffee.dev\',\n \'image\': \'https://www.gravatar.com/avatar/7ecdc5e1441ecd501faaf42a6ab9d6c0?s=200\',\n \'scrape\': False,\n \'title\': \'Ameya Shenoy\',\n \'type\': \'website\',\n \'url\': \'https://codingcoffee.dev\'}\n```\n\nUse `scrape=True` to fetch data using scraping instead of relying on open graph tags\n\n```sh\n>>> link = sneakpeek.SneakPeek(url="https://news.ycombinator.com/item?id=23812063", scrape=True)\n>>> link.fetch()\n>>> pprint(link)\n{\'description\': \'\',\n \'domain\': \'news.ycombinator.com\',\n \'image\': \'y18.gif\',\n \'scrape\': True,\n \'title\': \'WireGuard as VPN Server on Kubernetes with AdBlocking | Hacker News\',\n \'type\': \'other\',\n \'url\': \'https://news.ycombinator.com/item?id=23812063\'}\n ```\n\n### From HTML\n\n```\n>>> HTML = """\n... <html xmlns:og="http://ogp.me/ns">\n... <head>\n... <title>The Rock (1996)</title>\n... <meta property="og:title" content="The Rock" />\n... <meta property="og:description" content="The Rock: Directed by Michael Bay. With Sean Connery, Nicolas Cage, Ed Harris, John Spencer. A mild-mannered chemist and an ex-con must lead the counterstrike when a rogue group of military men, led by a renegade general, threaten a nerve gas attack from Alcatraz against San Francisco.">\n... <meta property="og:type" content="movie" />\n... <meta property="og:url" content="http://www.imdb.com/title/tt0117500/" />\n... <meta property="og:image" content="https://m.media-amazon.com/images/M/MV5BZDJjOTE0N2EtMmRlZS00NzU0LWE0ZWQtM2Q3MWMxNjcwZjBhXkEyXkFqcGdeQXVyNDk3NzU2MTQ@._V1_FMjpg_UX1000_.jpg">\n... </head>\n... </html>\n... """\n>>> movie = sneakpeek.SneakPeek(html=HTML)\n>>> movie.is_valid()\nTrue\n>>> pprint(movie)\n{\'description\': \'The Rock: Directed by Michael Bay. With Sean Connery, Nicolas \'\n                \'Cage, Ed Harris, John Spencer. A mild-mannered chemist and an \'\n                \'ex-con must lead the counterstrike when a rogue group of \'\n                \'military men, led by a renegade general, threaten a nerve gas \'\n                \'attack from Alcatraz against San Francisco.\',\n \'domain\': None,\n \'image\': \'https://m.media-amazon.com/images/M/MV5BZDJjOTE0N2EtMmRlZS00NzU0LWE0ZWQtM2Q3MWMxNjcwZjBhXkEyXkFqcGdeQXVyNDk3NzU2MTQ@._V1_FMjpg_UX1000_.jpg\',\n \'scrape\': False,\n \'title\': \'The Rock\',\n \'type\': \'movie\',\n \'url\': \'http://www.imdb.com/title/tt0117500/\'}\n```\n\n\n## Usage as a Server\n\nA simple server using FastAPI and uvicorn is used to serve the requests.\n\n```sh\nsneekpeek serve\n```\n\nYou can view the docs at http://localhost:9000/docs\n\n\n## Usage as a CLI\n\n```\nsneakpeek preview --url "https://github.com/codingcoffee/" | jq\n{\n  "domain": "github.com",\n  "scrape": false,\n  "url": "https://github.com/codingCoffee",\n  "title": "codingCoffee - Overview",\n  "type": "profile",\n  "image": "https://avatars.githubusercontent.com/u/13611153?v=4?s=400",\n  "description": "Automate anything and everything 🙋\u200d♂️. codingCoffee has 68 repositories available. Follow their code on GitHub.",\n  "error": null,\n  "image:alt": "Automate anything and everything 🙋\u200d♂️. codingCoffee has 68 repositories available. Follow their code on GitHub.",\n  "site_name": "GitHub"\n}\n```\n\n## Docker\n\n### As a Server\n\n```sh\ndocker run -it --rm -p 9000:9000 codingcoffee/sneakpeek -- serve --host 0.0.0.0\n```\n\n### As a CLI\n\n```sh\ndocker run -it --rm -p 9000:9000 codingcoffee/sneakpeek -- preview --url "https://github.com/codingcoffee"\n```\n\n\n## Configuration\n\n### Twitter\n\n- Sign up for a developer account on twitter [here](https://developer.twitter.com/)\n- Create an app\n- Add the following variables as ENV vars\n\n\n```\nTWITTER_CONSUMER_KEY="sample"\nTWITTER_CONSUMER_SECRET="sample"\nTWITTER_ACCESS_TOKEN="sample"\nTWITTER_ACCESS_TOKEN_SECRET="sample"\n```\n\n\n## Development\n\n```\npip install -U poetry\ngit clone https://github.com/codingcoffee/sneakpeek\ncd sneakpeek\npoetry install\n```\n\n\n## Running Tests\n\n```sh\npoetry run pytest\n```\n\n- Tested Websites\n  - [x] [YouTube](https://youtube.com)\n  - [x] [GitHub](https://github.com)\n  - [x] [LinkedIN](https://linkedin.com)\n  - [x] [Reddit](https://reddit.com)\n  - [x] [StackOverflow](https://stackoverflow.com)\n  - [x] [Business Insider](https://www.businessinsider.in)\n  - [x] [HackerNews](https://news.ycombinator.com/)\n  - [x] [Twitter](https://twitter.com)\n\n\n## TODO\n\n- [ ] [Instagram](https://instagram.com) (using [instagram-scraper](https://github.com/arc298/instagram-scraper))\n- [ ] [Facebook](https://facebook.com)\n- [ ] CI/CD for publishing to PyPi\n\n\n## Contribution\n\nHave better suggestions to optimize the server image? Found some typos? Need special handling for a new website? Found a bug? Go ahead and create an [Issue](https://github.com/codingcoffee/sneakpeek/issues)! Contributions of any kind welcome!\n\nWant to work on a TODO? Its always a good idea to talk about what are going to do before you actually start it, so frustration can be avoided.\n\nSome rules for coding:\n\n- Use the code style the project uses\n- For each feature, make a seperate branch, so it can be reviewed separately\n- Use commits with a good description, so everyone can see what you did\n\n\n## License\n\nThe code in this repository has been released under the [MIT License](https://opensource.org/licenses/MIT)\n\n',
    'author': 'Ameya Shenoy',
    'author_email': 'shenoy.ameya@gmail.com',
    'maintainer': 'Ameya Shenoy',
    'maintainer_email': 'shenoy.ameya@gmail.com',
    'url': 'https://github.com/codingcoffee/sneakpeek',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""SneakPeek API."""

# third-party imports
from fastapi import FastAPI

# app imports
import sneakpeek

app = FastAPI(title="SneakPeek API")


@app.get("/")
def get_link_preview(url: str = None, html: str = None, scrape: bool = False):
    link = sneakpeek.SneakPeek(url=url, html=html, scrape=scrape)
    if url:
        link.fetch()
    return dict(link)


@app.get("/version")
def version():
    return {"sneakpeek": {"version": sneakpeek.__version__}}

